%% Ejemplo de Correlacin para Seales de Energia.

%// Autor: SP
%// GUI para visualizar correlacion deterministica de seales. Ejecutar
%// desde el command window de esta manera: >> GUIcorrelacion

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% No hace falta entender toda esta funcion, si solo la quieren usar hay que 
% modificar las funciones x(t) e y(t) mas abajo para implementar 
% las funciones que ustedes quieran 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% La siguiente funcion genera la Interfaz Grafica (GUI) e inicializa todos sus elementos

function GUIcorrelacion
figure('units','normalized','outerposition',[0.1 0.1 .8 .8])        % Crea la figura 

% Grafico con las dos funciones a correlacionar
subplot(3,1,1)
dt=0.01;
t = -2:dt:2;                                                      % vector de variable independiente, en este script debe ser simetrico [-T,T]
M0=min(min(y(t))-.1*abs(min(y(t))),min(x(t))-.1*abs(min(x(t))));    % Para fijar los limites delgrafico automaticamente un 10% mas que las funciones
M1=max(max(y(t))+.1*abs(max(y(t))),max(y(t))+.1*abs(max(y(t))));    % Para fijar los limites delgrafico automaticamente un 10% mas que las funciones
[intmin,intmax]=intervalo(t,y(t),x(t));                             % Intervalo donde las funciones son distintas de cero

hplot4=fill([intmin,intmax,intmax,intmin],[M0 M0 M1 M1],[.7 .7 .7],'EdgeColor','none');hold on; % Sombreado del intervalo previo
plot(t,y(t),'k'); ylim([M0 M1]);xlim([min(t) max(t)]);              % Grafica la seal y(t)en negro, fijo los limites antes calculados para que se vea lindo
hplot = plot(t,x(t));                                               % Grafica la seal x(t+tau) en azul
htext=text(min(t)*.9,M1*.8,'$\tau=0$','interpreter','latex','Fontsize',15);  % Escribe el texto tau=valor en la figura
xlabel('t','interpreter','latex','Fontsize',15)                   % labels
ylabel('x(t+$\tau$) $\qquad$ y(t)','interpreter','latex','Fontsize',15)


% Grafico con las dos funciones multiplicadas
subplot(3,1,2)
[txxx,tyyy]=meshgrid(t,t);
M2=min(min(y(txxx).*x(txxx+tyyy))); M2=M2-.1*abs(M2);                %Para fijar los limites delgrafico automaticamente un 10% mas que las funciones
M3=max(max(y(txxx).*x(txxx+tyyy))); M3=M3+.1*abs(M3);                %Para fijar los limites delgrafico automaticamente un 10% mas que las funciones
hplot3= area(t,y(t).*x(t),'FaceColor',[.7 .7 .7]); 
ylim([M2 M3]);xlim([min(t) max(t)]); hold on; 
% Grafica la multiplicacion de las seales y sombrea el area a integrar, fijo los limites antes calculados para que se vea lindo
xlabel('t','interpreter','latex','Fontsize',15)
ylabel('x(t+$\tau$)y(t)','interpreter','latex','Fontsize',15)


% Grafico con la correlacion
subplot(3,1,3)
M4=min(conv(y(-t),x(t),'same')*dt); M4=M4-.1*abs(M4);         % Usamos la funcion conv que calcula la convolucin para calcular la correlacin
M5=max(conv(y(-t),x(t),'same')*dt); M5=M5+.1*abs(M5);         % Como usamos 'same', t debe ser simetrico para marcar el cero

plot(t,conv(y(-t),x(t),'same')*dt,'k');                       % Grafica la correlacin Rxy(tau), si queremos Ryx ver ej 3aI
ylim([M4 M5]);xlim([min(t) max(t)]); hold on;

hplot2 = plot([0 0],[M4 M5],'linewidth',2);                          % Grafica la recta vertical para mostrar que tau estamos evaluando en las graficas anteriores
xlabel('$\tau$','interpreter','latex','Fontsize',15)
ylabel('$r_{xy}(\tau)$','interpreter','latex','Fontsize',15)

% Genera el slider para controlar el valor de tau en las graficas, el valor
% inicial es cero
h = uicontrol('style','slider','Value',0,'Min',t(1),'Max',t(end),'units','normalized','position',[.12 0.02 .8 .04]);
% Le agregamos el callback al slider. Esto es la funcion que se llama cada
% vez que se mueve y cambia el valor del slider, a la funcion le pasamos
% todos los graficos para poder actualizarlos
addlistener(h,'Action',@(hObject, ~) makeplot(hObject, t,hplot,htext,hplot2,hplot3,hplot4));

% Dependiendo de la version de Matlab si tienen problemas cambiar
% Action por ContinuousValueChange o por ActionEvent
% Incluso si usan la ultima version, 2017, hay formas ms simples de
% implementarlo
end

function makeplot(hObject,t,hplot,htext,hplot2,hplot3,hplot4)
% funcion callback que actualiza todos los graficos en tiempo real

n = get(hObject,'Value');                   % Obtenemos el valor del slider
set(hplot,'ydata',x(t+n));                  % Actualizamos y(t+tau)
set(htext,'string',['$\tau$=' num2str(n)])	% Actualizamos el valor de tau en el texto
set(hplot2,'xdata',[n n]);                  % Actualizamos la recta vertical 
set(hplot3,'ydata',y(t).*x(t+n));           % Actualizamos el producto de las dos funciones


% Actualizamos el rectangulo sombreado que indica la region a integrar en
% el primer grafico
v=get(hplot4,'Vertices');
[intmin,intmax]=intervalo(t,y(t),x(t+n));%M0=min(y(t))-1;M1=max(y(t))+1;
if ~(isempty(intmin) || isempty(intmin))
set(hplot4,'Vertices',[intmin,intmax,intmax,intmin;v(:,2)']')
end

% Todo lo anterior lo prepara y con este comando actualiza los graficos
drawnow;
end


% Funcion para calcular el intervalo donde el producto entre las funciones
% x e y es distinto de cero
function [x,y]=intervalo(t,x,y)
temp=t((x.*y~=0));
x=min(temp);
y=max(temp);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estas funciones son las que hay que modificar para probar otros
% ejercicios, son x(t) e y(t).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Funcion x(t)
% function y=x(t)
%     y=sin(2*pi*t).*(t>=0 & t<1);
% end
% 
% % Funcion y(t)
% function y=y(t)
%     y=cos(2*pi*t).*(t>=-.25 & t<.75);
% end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Ejemplos de funciones para los ejercicios, para usarlas comentar las de
% arriba y descomentar estas
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% function y=x(t)
% y=(1+t).*(t>=-1 & t<0);
% end
% 
% function y=y(t)
% y=ones(size(t)).*(t>=-.5 & t<.5);
% end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% AUTOCORRELACION: x e y son la misma funcin

% Ejercicio 3 d I 
% function y=x(t)
%     y=(t).*(t>=-.5 & t<.5);
% end
% function y=y(t)
%     y=x(t);
% end

% Ejemplo de autocorrelacion de un cajon. Notar que se obtiene una funcion
% triangulo.
% function y=x(t)
%     y=ones(size(t)).*(t>=-.5 & t<.5);
% end
% function y=y(t)
%     y=x(t);
% end

% Ejemplo de autocorrelacion de una funcin no simtrica
% function y=x(t)
%     y=(1+t).*(t>=-1 & t<1);
% end
% function y=y(t)
%     y=x(t);
% end

% Ejemplo de autocorrelacion de un sinc. Notar que se obtiene un sinc...
function y=x(t)
    y=sinc(t);
end
function y=y(t)
    y=x(t);
end

