%% Ejemplo de Correlacin Discreta para seales de Energia

%// Autor: SP
%// GUI para visualizar correlacion deterministica de seales. Ejecutar
%// desde el command window de esta manera: >> GUIcorrelacionDiscreta

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% No hace falta entender toda esta funcion, si solo la quieren usar hay que 
% modificar las funciones x[n] e y[n] mas abajo para implementar 
% las funciones que ustedes quieran 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% La siguiente funcion genera la Interfaz Grafica (GUI) e inicializa todos sus elementos

function GUIcorrelacionDiscreta
figure('units','normalized','outerposition',[0.1 0.1 .8 .8])        % Crea la figura 

% Grafico con las dos funciones a correlacionar
subplot(3,1,1)
t = -100:1:100;                                                     % vector de variable independiente, en este script debe ser simetrico [-T,T]
M0=min(min(y(t))-.1*abs(min(y(t))),min(x(t))-.1*abs(min(x(t))));    % Para fijar los limites delgrafico automaticamente un 10% mas que las funciones
M1=max(max(y(t))+.1*abs(max(y(t))),max(y(t))+.1*abs(max(y(t))));    % Para fijar los limites delgrafico automaticamente un 10% mas que las funciones
[intmin,intmax]=intervalo(t,y(t),x(t));                             % Intervalo donde las funciones son distintas de cero

hplot4=fill([intmin,intmax,intmax,intmin],[M0 M0 M1 M1],[.7 .7 .7],'EdgeColor','none');hold on; % Sombreado del intervalo previo
stem(t,y(t),'k'); ylim([M0 M1]);xlim([min(t) max(t)]);              % Grafica la seal y[n] en negro, fijo los limites antes calculados para que se vea lindo
hplot = stem(t,x(t));                                               % Grafica la seal x[n+m] en azul
htext=text(min(t)*.9,M1*.8,'m=0');                                  % Escribe el texto m=valor en la figura
xlabel('$n$','interpreter','latex','Fontsize',15)                   % labels
ylabel('x[n+m] $\qquad$ y[n]','interpreter','latex','Fontsize',15)


% Grafico con las dos funciones multiplicadas
subplot(3,1,2)
[txxx,tyyy]=meshgrid(t,t);
M2=min(min(y(txxx).*x(txxx+tyyy))); M2=M2-.1*abs(M2);               % Para fijar los limites delgrafico automaticamente un 10% mas que las funciones
M3=max(max(y(txxx).*x(txxx+tyyy))); M3=M3+.1*abs(M3);               % Para fijar los limites delgrafico automaticamente un 10% mas que las funciones
hplot3= stem(t,y(t).*x(t)); ylim([M2 M3]);xlim([min(t) max(t)]); hold on; 
% Grafica la multiplicacion de las seales, fijo los limites antes calculados para que se vea lindo
xlabel('$n$','interpreter','latex','Fontsize',15)
ylabel('x[n+m]y[n]','interpreter','latex','Fontsize',15)


% Grafico con la correlacion
subplot(3,1,3)
M4=min(conv(y(-t),x(t),'same')); M4=M4-.1*abs(M4);        % Usamos la funcion conv que calcula la convolucin para calcular la correlacin
M5=max(conv(y(-t),x(t),'same')); M5=M5+.1*abs(M5);        % Como usamos 'same', t debe ser simetrico para marcar el cero

stem(t,conv(y(-t),x(t),'same'),'k');                      % Grafica la correlacin rxy[m], si queremos ryx ver ej 3aI
ylim([M4 M5]);xlim([min(t) max(t)]); hold on;

hplot2 = plot([0 0],[M4 M5],'linewidth',2);                         % Grafica la recta vertical para mostrar que m estamos evaluando en las graficas anteriores
xlabel('m','interpreter','latex','Fontsize',15)
ylabel('$r_{xy}[m]$','interpreter','latex','Fontsize',15)

% Genera el slider para controlar el valor de m en las graficas, el valor
% inicial es cero
h = uicontrol('style','slider','Value',0,'Min',t(1),'Max',t(end),'units','normalized','position',[.12 0.02 .8 .04]);
% Le agregamos el callback al slider. Esto es la funcion que se llama cada
% vez que se mueve y cambia el valor del slider, a la funcion le pasamos
% todos los graficos para poder actualizarlos
addlistener(h,'Action',@(hObject, ~) makeplot(hObject, t,hplot,htext,hplot2,hplot3,hplot4));

% Dependiendo de la version de Matlab si tienen problemas cambiar
% Action por ContinuousValueChange o por ActionEvent
% Incluso si usan la ultima version, 2017, hay formas ms simples de
% implementarlo
end

function makeplot(hObject,t,hplot,htext,hplot2,hplot3,hplot4)
% funcion callback que actualiza todos los graficos en tiempo real

n = round(get(hObject,'Value'));        % Obtenemos el valor del slider y redondeamos a un entero
set(hplot,'ydata',x(t+n));              % Actualizamos x[n+m]
set(htext,'string',['m=' num2str(n)])	% Actualizamos el valor de m en el texto
set(hplot2,'xdata',[n n]);              % Actualizamos la recta vertical 
set(hplot3,'ydata',y(t).*x(t+n));       % Actualizamos el producto de las dos funciones


% Actualizamos el rectangulo sombreado que indica la region a integrar en
% el primer grafico
v=get(hplot4,'Vertices');
[intmin,intmax]=intervalo(t,y(t),x(t+n));%M0=min(y(t))-1;M1=max(y(t))+1;
if ~(isempty(intmin) || isempty(intmin))
set(hplot4,'Vertices',[intmin,intmax,intmax,intmin;v(:,2)']')
end

% Todo lo anterior lo prepara y con este comando actualiza los graficos
drawnow;
end


% Funcion para calcular el intervalo donde el producto entre las funciones
% x e y es distinto de cero
 function [x,y]=intervalo(t,x,y)
 temp=t((x.*y~=0));
 x=min(temp);
 y=max(temp);
 end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estas funciones son las que hay que modificar para probar otros
% ejercicios, son x[n] e y[n].
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Ejercicio 3dII
function y=x(t)
    a=.95;
    y=(a).^t.*(t>=0);
end
function y=y(t)
    y=x(t);
end

% Cajon Discreto de N (impar) puntos.
% function y=x(t)
%     N = 11;
%     y=1.*((t>=-N) & (t<=N));
% end
% function y=y(t)
%     y=x(t);
% end